/*

Copyright (C) 2002 - 2003, Takaya Sakusabe

Modified and extended 2006-2007
by Precision Diagnostic Systems, Inc.
Maple Park, IL 60151 USA


Distributed under the terms of the GNU General Public License.

*/
	//==============================
	//
	//
	//
	//==============================
	class BrowserCheck {

		//==============================
		//
		//==============================
		public readonly String type = null;
		public readonly Single version = 0.0F;
		public readonly String os = null;

		//==============================
		//
		//==============================
		public BrowserCheck(string hua) {

			if (hua.IndexOf("Mozilla/") < 0)
				return;

			Regex rex = new Regex("\\d+\\.\\d+");

			int ie = hua.IndexOf("MSIE");
			int op = hua.IndexOf("Opera");
			if (ie > 0) {
				this.type = "IE"; // Internet Expolorer

				String verStr = rex.Match(hua, ie + 4).Value;
				this.version = Single.Parse(verStr);

				if (hua.IndexOf("Mac_") > 0)
					this.os = "M";
				else
					this.os = "W";

			} else if (op > 0) {
				this.type = "OP"; // Opera Browser

				String verStr = rex.Match(hua, op + 6).Value;
				this.version = Single.Parse(verStr);

				this.os = hua.Substring(hua.IndexOf("(") + 1, 1);

			} else if (hua.IndexOf("Gecko") > 0) {
				this.type = "GE"; // Gecko Rendering Engine 

				int i = hua.IndexOf("rv:");
				if (i > 0) {
					String verStr = rex.Match(hua, i + 3).Value;
					this.version = Single.Parse(verStr);
				} else {
					this.version = 1.0F;
				}

				if (hua.IndexOf("X11") > 0)
					this.os = "X";
				else if (hua.IndexOf("Macintosh") > 0)
					this.os = "M";
				else
					this.os = "W";

			} else if (hua.IndexOf("compatible") < 0) {
				this.type = "NN"; // Netscape Navigator
		
				String verStr = rex.Match(hua, 8).Value;
				this.version = Single.Parse(verStr);
		
				if (hua.IndexOf("X11") > 0)
					this.os = "X";
				else if (hua.IndexOf("Macintosh") > 0)
					this.os = "M";
				else
					this.os = "W";
			}
		}

	}

	//==============================
	//
	//
	//
	//==============================
	class ClientFunctionNames {
		public readonly String getElementById;
		public readonly String innerText;
		public readonly String posTop, posLeft;

		public ClientFunctionNames(BrowserCheck browser) {
			switch (browser.type) {
			case "IE":
				getElementById = "all";
				innerText = "innerText";
				posTop = "posTop";
				posLeft = "posLeft";
				break;
			case "GE":
				getElementById = "getElementById";
				innerText = "innerHTML";
				posTop = "top";
				posLeft = "left";
				break;
			}
		}
	}

	//==============================
	//
	//==============================
	void setup(HttpRequest req) {
		m_req = req;

		m_browser = new BrowserCheck(req.UserAgent);

		m_cfn = new ClientFunctionNames(m_browser);
	}

	//==============================
	//
	//==============================
	static String normalizeDate(String date) {
		if (date == null)
			return String.Empty;
		StringBuilder sb = new StringBuilder(8);
		foreach (Char ch in date) {
			if (Char.IsDigit(ch))
				sb.Append(ch);
		}
		return sb.ToString();
	}
	//==============================
	//
	//==============================
	static String normalizeTime(String time) {
		if (time == null)
			return String.Empty;
		StringBuilder sb = new StringBuilder(16);
		foreach (Char ch in time) {
			if (Char.IsDigit(ch) || ch == '.')
				sb.Append(ch);
		}
		return sb.ToString();
	}

	//==============================
	//
	//==============================
	static String prettyDate(String date) {
		if (date == null)
			return String.Empty;
		else if (date.Length != 8)
			return date;
		else
			return date.Substring(0, 4) + '/' + date.Substring(4, 2) + '/' + date.Substring(6, 2);
	}
	//==============================
	//
	//==============================
	static String prettyTime(String time) {
		if (time == null)
			return String.Empty;
		else if (time.Length < 4)
			return time;
		else
			return time.Substring(0, 2) + ':' + time.Substring(2, 2);
	}

	//==============================
	//
	//==============================
	String get_opt_str(string name, string defaultValue) {
		String str = m_req.QueryString.Get(name);
		if (str != null)
			return str;
		else
			return defaultValue;
	}
	
	//==============================
	//
	//==============================
	Int32 get_opt_int(string name, int defaultValue) {
		String str = m_req.QueryString.Get(name);
		if (str != null)
			return System.Convert.ToInt32(str);
		else
			return defaultValue;
	}

	//==============================
	//
	//
	//
	//==============================
	class QueryStringValueBase {
		protected HttpRequest m_req;
		//==============================
		//
		//==============================
		protected String get_opt_str(String name, string defaultValue) {
			String str = m_req.QueryString.Get(name);
			if (str != null)
				return str;
			else
				return defaultValue;
		}
		
		//==============================
		//
		//==============================
		protected Int32 get_opt_int(String name, int defaultValue) {
			String str = m_req.QueryString.Get(name);
			if (str != null)
				return System.Convert.ToInt32(str);
			else
				return defaultValue;
		}

		//==============================
		//
		//==============================
		protected QueryStringValueBase(HttpRequest req) {
			m_req = req;
		}

		//==============================
		//
		//==============================
		protected String get_val(String label, StringDictionary pref, String defaultValue, String fallbackValue) {
			String val = get_opt_str(label, null);
			if (val != null)
				pref[label] = val;
			else if (pref[label] != null)
				val = pref[label];
			else if (defaultValue != null)
				val = defaultValue;
			else
				val = fallbackValue;
			return val;
		}

	}


	//==============================
	//
	//
	//==============================
	sealed class VRMismatchException : Exception {
		public readonly VR m_dataVR;
		public readonly VR m_requestedVR;

		public VRMismatchException()
			: base() {
		}

		public VRMismatchException(VR dataVR, VR requestedVR)
			: base()
		{
			this.m_dataVR = dataVR;
			this.m_requestedVR = requestedVR;
		}

		public override String Message {
			get {
				String msg = base.Message;
				msg += m_dataVR + " " + m_requestedVR;
				return msg;
			}
		}
	}

	//==============================
	// 
	//==============================
	static ArrayList get_data_array(DataElementObject deo, VR vr) {
		if (vr != VR.NONE && deo.vr != vr)
			throw new VRMismatchException(deo.vr, vr);
	
		if (deo.vm == 0) {
			return null;
		} else if (deo.vm == 1) {
			ArrayList al = new ArrayList(1);
			al.Add(deo.value);
			return al;
		} else {
			return (ArrayList)deo.value;
		}
	}
	//==============================
	// 
	//==============================
	static ArrayList get_data_array(SOPInstanceObject file, DicomStudy dom, UInt32 tag, VR vr) {
		if (file == null)
			file = (SOPInstanceObject)dom.fl[0];
		foreach (DataElementObject deo in file.data) {
			if (deo.tag < tag)
				continue;
			else if (deo.tag > tag)
				return null;
			else
				return get_data_array(deo, vr);
		}
		return null;
	}
	//==============================
	// 
	//==============================
	static ArrayList get_data_array(SOPInstanceObject file, DicomStudy dom, UInt32 tag) {
		return get_data_array(file, dom, tag, VR.NONE);
	}

	//==============================
	// 
	//==============================
	static Object get_data(DataElementObject deo, VR vr) {
		if (vr != VR.NONE && deo.vr != vr && deo.vm != 0)
			throw new VRMismatchException(deo.vr, vr);
	
		if (deo.vm == 0)
			return null;
		else if (deo.vm == 1)
			return deo.value;
		else
			return ((ArrayList)deo.value)[0];
	}
	//==============================
	// 
	//==============================
	static Object get_data(SOPInstanceObject file, DicomStudy dom, UInt32 tag, VR vr) {
		if (file == null)
			file = (SOPInstanceObject)dom.fl[0];
		foreach (DataElementObject deo in file.data) {
			if (deo.tag < tag)
				continue;
			else if (deo.tag > tag)
				return null;
			else
				return get_data(deo, vr);
		}
		return null;
	}

	//==============================
	// 
	//==============================
	static Object get_data(SOPInstanceObject file, DicomStudy dom, UInt32 tag) {
		return get_data(file, dom, tag, VR.NONE);
	}

	//==============================
	// 
	//==============================
	//==============================
	static String get_data_AE(SOPInstanceObject file, DicomStudy dom, UInt32 tag, String defaultValue) {
		Object data = get_data(file, dom, tag, VR.AE);
		if (data == null)
			return defaultValue;
		else
			return (String)data;
	}
	//==============================
	static String get_data_AS(SOPInstanceObject file, DicomStudy dom, UInt32 tag, String defaultValue) {
		Object data = get_data(file, dom, tag, VR.AS);
		if (data == null)
			return defaultValue;
		else
			return (String)data;
	}
	//==============================
	static String get_data_CS(SOPInstanceObject file, DicomStudy dom, UInt32 tag, String defaultValue) {
		Object data = get_data(file, dom, tag, VR.CS);
		if (data == null)
			return defaultValue;
		else
			return (String)data;
	}
	//==============================
	static String get_data_DA(SOPInstanceObject file, DicomStudy dom, UInt32 tag, String defaultValue) {
		Object data = get_data(file, dom, tag, VR.DA);
		if (data == null)
			return defaultValue;
		else
			return (String)data;
	}
	//==============================
	static String get_data_DS(SOPInstanceObject file, DicomStudy dom, UInt32 tag, String defaultValue) {
		Object data = get_data(file, dom, tag, VR.DS);
		if (data == null)
			return defaultValue;
		else
			return (String)data;
	}
	//==============================
	static String get_data_DT(SOPInstanceObject file, DicomStudy dom, UInt32 tag, String defaultValue) {
		Object data = get_data(file, dom, tag, VR.DT);
		if (data == null)
			return defaultValue;
		else
			return (String)data;
	}
	//==============================
	static String get_data_IS(SOPInstanceObject file, DicomStudy dom, UInt32 tag, String defaultValue) {
		Object data = get_data(file, dom, tag, VR.IS);
		if (data == null)
			return defaultValue;
		else
			return (String)data;
	}
	//==============================
	static String get_data_LO(SOPInstanceObject file, DicomStudy dom, UInt32 tag, String defaultValue) {
		Object data = get_data(file, dom, tag, VR.LO);
		if (data == null)
			return defaultValue;
		else
			return (String)data;
	}
	//==============================
	static String get_data_LT(SOPInstanceObject file, DicomStudy dom, UInt32 tag, String defaultValue) {
		Object data = get_data(file, dom, tag, VR.LT);
		if (data == null)
			return defaultValue;
		else
			return (String)data;
	}
	//==============================
	static String get_data_PN(SOPInstanceObject file, DicomStudy dom, UInt32 tag, String defaultValue) {
		Object data = get_data(file, dom, tag, VR.PN);
		if (data == null)
			return defaultValue;
		else
			return (String)data;
	}
	//==============================
	static String get_data_SH(SOPInstanceObject file, DicomStudy dom, UInt32 tag, String defaultValue) {
		Object data = get_data(file, dom, tag, VR.SH);
		if (data == null)
			return defaultValue;
		else
			return (String)data;
	}
	//==============================
	static Int32 get_data_SL(SOPInstanceObject file, DicomStudy dom, UInt32 tag, Int32 defaultValue) {
		Object data = get_data(file, dom, tag, VR.SL);
		if (data == null)
			return defaultValue;
		else
			return (Int32)data;
	}
	//==============================
	static Int16 get_data_SS(SOPInstanceObject file, DicomStudy dom, UInt32 tag, Int16 defaultValue) {
		Object data = get_data(file, dom, tag, VR.SS);
		if (data == null)
			return defaultValue;
		else
			return (Int16)data;
	}
	//==============================
	static String get_data_ST(SOPInstanceObject file, DicomStudy dom, UInt32 tag, String defaultValue) {
		Object data = get_data(file, dom, tag, VR.ST);
		if (data == null)
			return defaultValue;
		else
			return (String)data;
	}
	//==============================
	static String get_data_TM(SOPInstanceObject file, DicomStudy dom, UInt32 tag, String defaultValue) {
		Object data = get_data(file, dom, tag, VR.TM);
		if (data == null)
			return defaultValue;
		else
			return (String)data;
	}
	//==============================
	static String get_data_UI(SOPInstanceObject file, DicomStudy dom, UInt32 tag, String defaultValue) {
		Object data = get_data(file, dom, tag, VR.UI);
		if (data == null)
			return defaultValue;
		else
			return (String)data;
	}
	//==============================
	static UInt32 get_data_UL(SOPInstanceObject file, DicomStudy dom, UInt32 tag, UInt32 defaultValue) {
		Object data = get_data(file, dom, tag, VR.UL);
		if (data == null)
			return defaultValue;
		else
			return (UInt32)data;
	}
	//==============================
	static UInt16 get_data_US(SOPInstanceObject file, DicomStudy dom, UInt32 tag, UInt16 defaultValue) {
		Object data = get_data(file, dom, tag, VR.US);
		if (data == null)
			return defaultValue;
		else
			return (UInt16)data;
	}
	//==============================
	static String get_data_UT(SOPInstanceObject file, DicomStudy dom, UInt32 tag, String defaultValue) {
		Object data = get_data(file, dom, tag, VR.UT);
		if (data == null)
			return defaultValue;
		else
			return (String)data;
	}

	//==============================
	//
	//==============================
	BrowserCheck m_browser = null;
	HttpRequest m_req = null;
	ClientFunctionNames m_cfn = null;

	//==============================
	//
	//==============================
	const NumberStyles DS_ns = NumberStyles.Float;
	const NumberStyles IS_ns = NumberStyles.Integer;
